# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import timedelta

from oslo_log import log

from almanach.core.controllers import base_controller
from almanach.core import exception
from almanach.core import model

LOG = log.getLogger(__name__)


class VolumeController(base_controller.BaseController):

    def __init__(self, config, database_adapter):
        self.database_adapter = database_adapter
        self.volume_existence_threshold = timedelta(0, config.entities.volume_existence_threshold)

    def list_volumes(self, project_id, start, end):
        return self.database_adapter.get_all_entities_by_project(project_id, start, end, model.Volume.TYPE)

    def create_volume(self, volume_id, project_id, start, volume_type, size, volume_name, attached_to=None):
        start = self._validate_and_parse_date(start)
        LOG.info("volume %s created in project %s to size %s on %s", volume_id, project_id, size, start)
        if self._fresher_entity_exists(volume_id, start):
            return

        volume_type_name = self._get_volume_type_name(volume_type)

        entity = model.Volume(volume_id, project_id, start, None, volume_type_name, size, start, volume_name,
                              attached_to)
        self.database_adapter.insert_entity(entity)

    def detach_volume(self, volume_id, date, attachments):
        date = self._validate_and_parse_date(date)
        LOG.info("volume %s detached on %s", volume_id, date)
        try:
            self._volume_detach_instance(volume_id, date, attachments)
        except exception.EntityNotFoundException as e:
            LOG.error("Trying to detach a volume with id '%s' not in the database yet.", volume_id)
            raise e

    def attach_volume(self, volume_id, date, attachments):
        date = self._validate_and_parse_date(date)
        LOG.info("Volume %s attached to %s on %s", volume_id, attachments, date)
        try:
            self._volume_attach_instance(volume_id, date, attachments)
        except exception.EntityNotFoundException as e:
            LOG.error("Trying to attach a volume with id '%s' not in the database yet.", volume_id)
            raise e

    def rename_volume(self, volume_id, volume_name):
        try:
            volume = self.database_adapter.get_active_entity(volume_id)
            if volume and volume.name != volume_name:
                LOG.info("volume %s renamed from %s to %s", volume_id, volume.name, volume_name)
                volume.name = volume_name
                self.database_adapter.update_active_entity(volume)
        except exception.EntityNotFoundException:
            LOG.error("Trying to update a volume with id '%s' not in the database yet.", volume_id)

    def resize_volume(self, volume_id, size, update_date):
        update_date = self._validate_and_parse_date(update_date)
        try:
            volume = self.database_adapter.get_active_entity(volume_id)
            LOG.info("volume %s updated in project %s to size %s on %s",
                     volume_id, volume.project_id, size, update_date)

            self.database_adapter.close_active_entity(volume_id, update_date)

            volume.size = size
            volume.start = update_date
            volume.end = None
            volume.last_event = update_date
            self.database_adapter.insert_entity(volume)
        except exception.EntityNotFoundException as e:
            LOG.error("Trying to update a volume with id '%s' not in the database yet.", volume_id)
            raise e

    def delete_volume(self, volume_id, delete_date):
        delete_date = self._localize_date(self._validate_and_parse_date(delete_date))
        LOG.info("volume %s deleted on %s", volume_id, delete_date)
        try:
            if self.database_adapter.count_entity_entries(volume_id) > 1:
                volume = self.database_adapter.get_active_entity(volume_id)
                if delete_date - volume.start < self.volume_existence_threshold:
                    self.database_adapter.delete_active_entity(volume_id)
                    return
            self.database_adapter.close_active_entity(volume_id, delete_date)
        except exception.EntityNotFoundException as e:
            LOG.error("Trying to delete a volume with id '%s' not in the database yet.", volume_id)
            raise e

    def _volume_attach_instance(self, volume_id, date, attachments):
        volume = self.database_adapter.get_active_entity(volume_id)
        date = self._localize_date(date)
        volume.last_event = date
        existing_attachments = volume.attached_to
        volume.attached_to = attachments

        if existing_attachments or self._is_within_threshold(date, volume):
            self.database_adapter.update_active_entity(volume)
        else:
            self._close_volume(volume_id, volume, date)

    def _volume_detach_instance(self, volume_id, date, attachments):
        volume = self.database_adapter.get_active_entity(volume_id)
        date = self._localize_date(date)
        volume.last_event = date
        volume.attached_to = attachments

        if attachments or self._is_within_threshold(date, volume):
            self.database_adapter.update_active_entity(volume)
        else:
            self._close_volume(volume_id, volume, date)

    def _close_volume(self, volume_id, volume, date):
        self.database_adapter.close_active_entity(volume_id, date)
        volume.start = date
        volume.end = None
        self.database_adapter.insert_entity(volume)

    def _is_within_threshold(self, date, volume):
        return date - volume.start < self.volume_existence_threshold

    def _get_volume_type_name(self, volume_type_id):
        if volume_type_id is None:
            return None

        volume_type = self.database_adapter.get_volume_type(volume_type_id)
        return volume_type.volume_type_name



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils

from almanach.tests.tempest.tests.api import base


class TestServerResize(base.BaseAlmanachTest):

    def setUp(self):
        super(base.BaseAlmanachTest, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(TestServerResize, cls).resource_setup()

    def test_instance_resize(self):
        tenant_id = uuidutils.generate_uuid()
        server = self.get_server_creation_payload()
        self.create_server_through_api(tenant_id, server)

        resize_data = {'date': '2016-01-01T18:40:00Z',
                       'flavor': 'resized_flavor'}
        serialized_data = json.dumps(resize_data)

        self.almanach_client.resize(server['id'], serialized_data)

        resp, response_body = self.almanach_client.get_tenant_entities(tenant_id)

        entities = json.loads(response_body)
        self.assertIsInstance(entities, list)
        self.assertEqual(2, len(entities))

        initial_server, resized_server = sorted(entities,
                                                key=lambda k: k['flavor'] if k['flavor'] == 'resized_flavor' else '')

        self.assertEqual(server['id'], initial_server['entity_id'])
        self.assertEqual(server['flavor'], initial_server['flavor'])
        self.assertEqual(server['id'], resized_server['entity_id'])
        self.assertEqual(resize_data['flavor'], resized_server['flavor'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.collector.handlers import volume_handler
from almanach.tests.unit import base
from almanach.tests.unit.builders import notification as builder


class TestVolumeHandler(base.BaseTestCase):

    def setUp(self):
        super(TestVolumeHandler, self).setUp()
        self.controller = mock.Mock()
        self.volume_handler = volume_handler.VolumeHandler(self.controller)

    def test_volume_created(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.create.end') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.create_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.payload['tenant_id'],
            notification.payload['created_at'],
            notification.payload['volume_type'],
            notification.payload['size'],
            notification.payload['display_name'],
        )

    def test_volume_deleted(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.delete.end') \
            .with_context_value('timestamp', 'a_date') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.delete_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.context['timestamp'],
        )

    def test_volume_renamed(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.update.end') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.rename_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.payload['display_name'],
        )

    def test_volume_renamed_with_exists_event(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.exists') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.rename_volume.assert_called_once_with(
                notification.payload['volume_id'],
                notification.payload['display_name'],
        )

    def test_volume_resized(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.resize.end') \
            .with_context_value('timestamp', 'a_date') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.resize_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.payload['size'],
            notification.context['timestamp'],
        )

    def test_volume_attach_empty(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.attach.end') \
            .with_context_value('timestamp', 'a_date') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.attach_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.context['timestamp'],
            []
        )

    def test_volume_attach_with_instances(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.attach.end') \
            .with_context_value('timestamp', 'a_date') \
            .with_instance_attached('instance_id1') \
            .with_instance_attached('instance_id2') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.attach_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.context['timestamp'],
            ['instance_id1', 'instance_id2']
        )

    def test_volume_detached(self):
        notification = builder.VolumeNotificationBuilder() \
            .with_event_type('volume.detach.end') \
            .with_context_value('timestamp', 'a_date') \
            .build()

        self.volume_handler.handle_events(notification)

        self.controller.detach_volume.assert_called_once_with(
            notification.payload['volume_id'],
            notification.context['timestamp'],
            []
        )



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.collector.handlers import instance_handler
from almanach.collector.handlers import volume_handler
from almanach.collector.handlers import volume_type_handler
from almanach.collector import notification
from almanach.collector import service

from almanach.tests.unit import base


class TestServiceFactory(base.BaseTestCase):

    def setUp(self):
        super(TestServiceFactory, self).setUp()
        self.core_factory = mock.Mock()
        self.factory = service.ServiceFactory(self.config, self.core_factory)

    def test_get_service(self):
        self.assertIsInstance(self.factory.get_service(),
                              service.CollectorService)

        self.core_factory.get_instance_controller.assert_called_once()
        self.core_factory.get_volume_controller.assert_called_once()
        self.core_factory.get_volume_type_controller.assert_called_once()

    def test_get_instance_handler(self):
        self.assertIsInstance(self.factory.get_instance_handler(),
                              instance_handler.InstanceHandler)

    def test_get_volume_handler(self):
        self.assertIsInstance(self.factory.get_volume_handler(),
                              volume_handler.VolumeHandler)

    def test_get_volume_type_handler(self):
        self.assertIsInstance(self.factory.get_volume_type_handler(),
                              volume_type_handler.VolumeTypeHandler)

    def test_get_on_delete_filters(self):
        self.assertIsInstance(self.factory.get_on_delete_filters(),
                              notification.NotificationFilter)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.collector.handlers import base_handler


class VolumeTypeHandler(base_handler.BaseHandler):

    def __init__(self, controller):
        super(VolumeTypeHandler, self).__init__()
        self.controller = controller

    def handle_events(self, notification):
        if notification.event_type == "volume_type.create":
            self._on_volume_type_created(notification)

    def _on_volume_type_created(self, notification):
        volume_types = notification.payload.get("volume_types")
        volume_type_id = volume_types.get("id")
        volume_type_name = volume_types.get("name")
        self.controller.create_volume_type(volume_type_id, volume_type_name)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.core.controllers import base_controller
from almanach.core import exception
from almanach.validators import instance_validator

LOG = log.getLogger(__name__)


class EntityController(base_controller.BaseController):

    def __init__(self, database_adapter):
        self.database_adapter = database_adapter

    def update_active_instance_entity(self, instance_id, **kwargs):
        try:
            instance_validator.InstanceValidator().validate_update(kwargs)
            instance = self.database_adapter.get_active_entity(instance_id)
            self._update_instance_object(instance, **kwargs)
            self.database_adapter.update_active_entity(instance)
            return instance
        except exception.EntityNotFoundException as e:
            LOG.error("Instance %s is not in the database yet.", instance_id)
            raise e

    def update_inactive_entity(self, instance_id, start, end, **kwargs):
        inactive_entities = self.database_adapter.get_all_entities_by_id_and_date(instance_id, start, end)
        if len(inactive_entities) > 1:
            raise exception.MultipleEntitiesMatchingQueryException()
        if len(inactive_entities) < 1:
            raise exception.EntityNotFoundException(
                    "InstanceId: {0} Not Found with start".format(instance_id))
        entity = inactive_entities[0]
        entity_update = self._transform_attribute_to_match_entity_attribute(**kwargs)
        self.database_adapter.update_closed_entity(entity=entity, data=entity_update)
        start = entity_update.get('start') or start
        end = entity_update.get('end') or end
        return self.database_adapter.get_all_entities_by_id_and_date(instance_id, start, end)[0]

    def entity_exists(self, entity_id):
        return self.database_adapter.count_entity_entries(entity_id=entity_id) >= 1

    def get_all_entities_by_id(self, entity_id):
        if not self.entity_exists(entity_id=entity_id):
            raise exception.EntityNotFoundException("Entity not found")
        return self.database_adapter.get_all_entities_by_id(entity_id=entity_id)

    def list_entities(self, project_id, start, end):
        return self.database_adapter.get_all_entities_by_project(project_id, start, end)

    def _update_instance_object(self, instance, **kwargs):
        for key, value in self._transform_attribute_to_match_entity_attribute(**kwargs).items():
            setattr(instance, key, value)
            LOG.info("Updating entity for instance '%s' with %s=%s", instance.entity_id, key, value)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class NotificationFilter(object):

    def __init__(self):
        self.filters = []

    def add(self, notification_filter):
        self.filters.append(notification_filter)

    def ignore_notification(self, notification):
        return any([f.ignore_notification(notification) for f in self.filters])


class NotificationMessage(object):
    RETRY_COUNTER = 'retry_count'

    def __init__(self, event_type, context, payload, metadata):
        self.event_type = event_type
        self.context = context
        self.payload = payload
        self.metadata = metadata

    def increment_retry_count(self):
        if self.has_counter():
            self.context[self.RETRY_COUNTER] += 1
        else:
            self.context[self.RETRY_COUNTER] = 1

    def get_retry_counter(self):
        if self.has_counter():
            return self.context[self.RETRY_COUNTER]
        return 0

    def has_counter(self):
        return self.RETRY_COUNTER in self.context


class NotificationHandler(object):

    def __init__(self, config, messaging):
        self.config = config
        self.messaging = messaging
        self.handlers = []

    def add_event_handler(self, handler):
        self.handlers.append(handler)

    def info(self, ctxt, publisher_id, event_type, payload, metadata):
        LOG.debug('Received event "%s" from "%s" on info queue', event_type, publisher_id)
        notification = NotificationMessage(event_type, ctxt, payload, metadata)

        try:
            for handler in self.handlers:
                handler.handle_events(notification)
        except Exception as e:
            LOG.warning('Send notification "%s" to error queue', notification.metadata.get('message_id'))
            LOG.warning('Notification event_type: %s', notification.event_type)
            LOG.warning('Notification context: %s', notification.context)
            LOG.warning('Notification payload: %s', notification.payload)
            LOG.warning('Notification metadata: %s', notification.metadata)
            LOG.exception(e)
            self._send_notification_to_error_queue(notification)

    def error(self, ctxt, publisher_id, event_type, payload, metadata):
        LOG.warning('Received event "%s" from "%s" on error queue', event_type, publisher_id)
        notification = NotificationMessage(event_type, ctxt, payload, metadata)

        if notification.has_counter():
            self._send_notification_to_info_queue(notification)
        else:
            LOG.info('Ignoring notification "%s" sent on error queue', event_type)

    def _send_notification_to_error_queue(self, notification):
        notifier = self.messaging.get_notifier()
        notification.increment_retry_count()

        if notification.get_retry_counter() > self.config.collector.max_retries:
            LOG.critical('Send notification "%s" (%s) to critical queue',
                         notification.metadata.get('message_id'),
                         notification.event_type)
            notifier.critical(notification.context, notification.event_type, notification.payload)
        else:
            notifier.error(notification.context, notification.event_type, notification.payload)

    def _send_notification_to_info_queue(self, notification):
        LOG.error('Retry notification "%s" on info queue in %s seconds',
                  notification.event_type,
                  self.config.collector.retry_delay)

        time.sleep(self.config.collector.retry_delay)

        notifier = self.messaging.get_notifier()
        notifier.info(notification.context, notification.event_type, notification.payload)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.tests.tempest.tests.scenario import base


class TestServerSuppressionScenario(base.BaseAlmanachScenarioTest):

    def test_delete_server(self):
        server, flavor = self.create_test_server(wait_until='ACTIVE')
        self.delete_test_server(server)
        self.wait_for_notification(self._check_entity_is_closed, server)

        entities = self.get_tenant_entities(server['tenant_id'])
        self.assertEqual(1, len(entities))
        self.assertEqual(server['id'], entities[0]['entity_id'])
        self.assertEqual('instance', entities[0]['entity_type'])
        self.assertEqual(server['name'], entities[0]['name'])
        self.assertEqual(flavor['name'], entities[0]['flavor'])
        self.assertIsNotNone(entities[0]['start'])
        self.assertIsNotNone(entities[0]['end'])

    def _check_entity_is_closed(self, server):
        entities = self.get_tenant_entities(server['tenant_id'])
        return len(entities) == 1 and entities[0]['end'] is not None



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from oslo_log import log as logging
from oslo_service import service

from almanach.collector import service as collector_service
from almanach.core import factory as core_factory
from almanach.core import opts


def main():
    opts.CONF(sys.argv[1:], project=opts.DOMAIN)
    logging.setup(opts.CONF, opts.DOMAIN)
    config = opts.CONF

    service_factory = collector_service.ServiceFactory(config, core_factory.Factory(config))
    launcher = service.launch(config, service_factory.get_service())
    launcher.wait()

if __name__ == '__main__':
    main()



# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.collector.filters import delete_instance_before_completion_filter
from almanach.collector import notification

from almanach.tests.unit import base
from almanach.tests.unit.builders import notification as builder


class TestDeleteInstanceBeforeCompletionFilter(base.BaseTestCase):

    def setUp(self):
        super(TestDeleteInstanceBeforeCompletionFilter, self).setUp()
        self.filter = delete_instance_before_completion_filter.DeleteInstanceBeforeCompletionFilter(self.config)

    def test_with_instance_deleted_quickly(self):
        message = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.delete.end') \
            .with_payload_value('state', 'deleted') \
            .with_payload_value('created_at', '2017-02-05 11:25:01+00:00') \
            .with_payload_value('deleted_at', '2017-02-05T11:34:36.000000') \
            .with_context_value(notification.NotificationMessage.RETRY_COUNTER, 3) \
            .build()

        self.assertTrue(self.filter.ignore_notification(message))

    def test_when_message_was_never_retried(self):
        message = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.delete.end') \
            .with_payload_value('state', 'deleted') \
            .with_payload_value('created_at', '2017-02-05 11:25:01+00:00') \
            .with_payload_value('deleted_at', '2017-02-05T11:34:36.000000') \
            .build()

        self.assertFalse(self.filter.ignore_notification(message))

    def test_with_instance_deleted_later(self):
        message = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.delete.end') \
            .with_payload_value('state', 'deleted') \
            .with_payload_value('created_at', '2017-02-05 11:25:01+00:00') \
            .with_payload_value('deleted_at', '2017-02-05T11:44:36.000000') \
            .with_context_value(notification.NotificationMessage.RETRY_COUNTER, 3) \
            .build()

        self.assertFalse(self.filter.ignore_notification(message))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.core import exception
from almanach.tests.unit.api.v1 import base_api
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import instance


class TestApiInstance(base_api.BaseApi):

    def test_get_instances(self):
        self.instance_ctl.list_instances.return_value = [a(instance().with_id('123'))]
        code, result = self.api_get('/project/TENANT_ID/instances',
                                    query_string={
                                        'start': '2014-01-01 00:00:00.0000',
                                        'end': '2014-02-01 00:00:00.0000'
                                    },
                                    headers={'X-Auth-Token': 'some token value'})

        self.instance_ctl.list_instances.assert_called_once_with(
            'TENANT_ID', base_api.a_date_matching("2014-01-01 00:00:00.0000"),
            base_api.a_date_matching("2014-02-01 00:00:00.0000")
        )
        self.assertEqual(code, 200)
        self.assertEqual(len(result), 1)
        self.assertIn('entity_id', result[0])
        self.assertEqual(result[0]['entity_id'], '123')

    def test_successful_instance_create(self):
        data = dict(id="INSTANCE_ID",
                    created_at="CREATED_AT",
                    name="INSTANCE_NAME",
                    flavor="A_FLAVOR",
                    os_type="AN_OS_TYPE",
                    os_distro="A_DISTRIBUTION",
                    os_version="AN_OS_VERSION")

        code, result = self.api_post(
            '/project/PROJECT_ID/instance',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.create_instance.assert_called_once_with(
            tenant_id="PROJECT_ID",
            instance_id=data["id"],
            create_date=data["created_at"],
            name=data['name'],
            flavor=data['flavor'],
            image_meta=dict(os_type=data['os_type'],
                            distro=data['os_distro'],
                            version=data['os_version'])
        )
        self.assertEqual(code, 201)

    def test_instance_create_missing_a_param_returns_bad_request_code(self):
        data = dict(id="INSTANCE_ID",
                    created_at="CREATED_AT",
                    name="INSTANCE_NAME",
                    flavor="A_FLAVOR",
                    os_type="AN_OS_TYPE",
                    os_version="AN_OS_VERSION")

        code, result = self.api_post(
            '/project/PROJECT_ID/instance',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.create_instance.assert_not_called()
        self.assertEqual(result["error"], "The 'os_distro' param is mandatory for the request you have made.")
        self.assertEqual(code, 400)

    def test_instance_create_bad_date_format_returns_bad_request_code(self):
        self.instance_ctl.create_instance.side_effect = exception.DateFormatException
        data = dict(id="INSTANCE_ID",
                    created_at="A_BAD_DATE",
                    name="INSTANCE_NAME",
                    flavor="A_FLAVOR",
                    os_type="AN_OS_TYPE",
                    os_distro="A_DISTRIBUTION",
                    os_version="AN_OS_VERSION")

        code, result = self.api_post(
            '/project/PROJECT_ID/instance',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.create_instance.assert_called_once_with(
            tenant_id="PROJECT_ID",
            instance_id=data["id"],
            create_date=data["created_at"],
            flavor=data['flavor'],
            image_meta=dict(os_type=data['os_type'],
                            distro=data['os_distro'],
                            version=data['os_version']),
            name=data['name']
        )

        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_successful_instance_resize(self):
        data = dict(date="UPDATED_AT",
                    flavor="A_FLAVOR")

        code, result = self.api_put(
            '/instance/INSTANCE_ID/resize',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.resize_instance.assert_called_once_with(
            instance_id="INSTANCE_ID",
            flavor=data['flavor'],
            resize_date=data['date']
        )
        self.assertEqual(code, 200)

    def test_successfull_instance_delete(self):
        data = dict(date="DELETE_DATE")

        code, result = self.api_delete('/instance/INSTANCE_ID', data=data, headers={'X-Auth-Token': 'some token value'})

        self.instance_ctl.delete_instance.assert_called_once_with(
            instance_id="INSTANCE_ID",
            delete_date=data['date']
        )
        self.assertEqual(code, 202)

    def test_instance_delete_missing_a_param_returns_bad_request_code(self):
        code, result = self.api_delete(
            '/instance/INSTANCE_ID',
            data=dict(),
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.delete_instance.assert_not_called()
        self.assertIn(
            "The 'date' param is mandatory for the request you have made.",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_instance_delete_no_data_bad_request_code(self):
        code, result = self.api_delete('/instance/INSTANCE_ID', headers={'X-Auth-Token': 'some token value'})

        self.instance_ctl.delete_instance.assert_not_called()
        self.assertIn(
            "Invalid parameter or payload",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_instance_delete_bad_date_format_returns_bad_request_code(self):
        data = dict(date="A_BAD_DATE")
        self.instance_ctl.delete_instance.side_effect = exception.DateFormatException

        code, result = self.api_delete('/instance/INSTANCE_ID', data=data, headers={'X-Auth-Token': 'some token value'})

        self.instance_ctl.delete_instance.assert_called_once_with(
            instance_id="INSTANCE_ID",
            delete_date=data['date']
        )
        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_instance_resize_missing_a_param_returns_bad_request_code(self):
        data = dict(date="UPDATED_AT")
        code, result = self.api_put(
            '/instance/INSTANCE_ID/resize',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.resize_instance.assert_not_called()
        self.assertIn(
            "The 'flavor' param is mandatory for the request you have made.",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_instance_resize_bad_date_format_returns_bad_request_code(self):
        self.instance_ctl.resize_instance.side_effect = exception.DateFormatException
        data = dict(date="A_BAD_DATE",
                    flavor="A_FLAVOR")
        code, result = self.api_put(
            '/instance/INSTANCE_ID/resize',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.resize_instance.assert_called_once_with(
            instance_id="INSTANCE_ID",
            flavor=data['flavor'],
            resize_date=data['date']
        )
        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_rebuild_instance(self):
        instance_id = 'INSTANCE_ID'
        data = {
            'distro': 'A_DISTRIBUTION',
            'version': 'A_VERSION',
            'os_type': 'AN_OS_TYPE',
            'rebuild_date': 'UPDATE_DATE',
        }
        code, result = self.api_put(
            '/instance/INSTANCE_ID/rebuild',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.rebuild_instance.assert_called_once_with(
            instance_id=instance_id,
            rebuild_date=data.get('rebuild_date'),
            image_meta=dict(distro=data.get('distro'),
                            version=data.get('version'),
                            os_type=data.get('os_type'))
        )
        self.assertEqual(code, 200)

    def test_rebuild_instance_missing_a_param_returns_bad_request_code(self):
        data = {
            'distro': 'A_DISTRIBUTION',
            'rebuild_date': 'UPDATE_DATE',
        }
        code, result = self.api_put(
            '/instance/INSTANCE_ID/rebuild',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.rebuild_instance.assert_not_called()
        self.assertIn(
            "The 'version' param is mandatory for the request you have made.",
            result["error"]
        )
        self.assertEqual(code, 400)

    def test_rebuild_instance_bad_date_format_returns_bad_request_code(self):
        self.instance_ctl.rebuild_instance.side_effect = exception.DateFormatException
        instance_id = 'INSTANCE_ID'
        data = {
            'distro': 'A_DISTRIBUTION',
            'version': 'A_VERSION',
            'os_type': 'AN_OS_TYPE',
            'rebuild_date': 'A_BAD_UPDATE_DATE',
        }
        code, result = self.api_put(
            '/instance/INSTANCE_ID/rebuild',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.instance_ctl.rebuild_instance.assert_called_once_with(
            instance_id=instance_id,
            rebuild_date=data.get('rebuild_date'),
            image_meta=dict(distro=data.get('distro'),
                            version=data.get('version'),
                            os_type=data.get('os_type'))
        )
        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.assertEqual(code, 400)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from tempest import config
from tempest.test_discover import plugins

import almanach
from almanach.tests.tempest import config as project_config


class AlmanachTempestPlugin(plugins.TempestPlugin):

    def load_tests(self):
        base_path = os.path.split(os.path.dirname(
                os.path.abspath(almanach.__file__)))[0]
        test_dir = "almanach/tests/tempest/tests"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    def register_opts(self, conf):
        config.register_opt_group(
                conf, project_config.service_available_group,
                project_config.ServiceAvailableGroup)
        config.register_opt_group(conf, project_config.usage_group,
                                  project_config.UsageGroup)

    def get_opt_lists(self):
        return [(project_config.usage_group.name,
                 project_config.UsageGroup)]



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_log import log as logging

CONF = cfg.CONF
DOMAIN = 'almanach'

database_opts = [
    cfg.StrOpt('driver',
               default='mongodb',
               help='Database driver'),
    cfg.StrOpt('connection_url',
               secret=True,
               default='mongodb://almanach:almanach@localhost:27017/almanach',
               help='Database connection URL'),
]

api_opts = [
    cfg.IPOpt('bind_ip',
              default='127.0.0.1',
              help='IP address to listen on'),
    cfg.PortOpt('bind_port',
                default=8000,
                help='TCP port number to listen on'),
]

collector_opts = [
    cfg.ListOpt('transport_url',
                secret=True,
                default='rabbit://guest:guest@localhost:5672',
                help='AMQP connection URL'),
    cfg.StrOpt('topic',
               default='almanach',
               help='AMQP topic used for OpenStack notifications'),
    cfg.IntOpt('max_retries',
               default=5,
               help='Number of retries before to send message to critical queue'),
    cfg.IntOpt('min_retries',
               default=3,
               help='Number of retries before to use filters to discard notifications'),
    cfg.IntOpt('retry_delay',
               default=25,
               help='Delay in seconds between retries'),
    cfg.IntOpt('thread_pool_size',
               default=1,
               help='Size of executor thread pool'),
]

keystone_opts = [
    cfg.StrOpt('username',
               help='Keystone service username'),
    cfg.StrOpt('password',
               secret=True,
               help='Keystone service password'),
    cfg.StrOpt('user_domain_id',
               default='default',
               help='Keystone service user domain ID'),
    cfg.StrOpt('user_domain_name',
               default='Default',
               help='Keystone service user domain name'),
    cfg.StrOpt('project_domain_name',
               default='Default',
               help='Keystone service project domain name'),
    cfg.StrOpt('project_name',
               default='service',
               help='Keystone service project name'),
    cfg.StrOpt('auth_url',
               default='http://127.0.0.1:35357/v3',
               help='Keystone API V3 admin endpoint'),
]

auth_opts = [
    cfg.StrOpt('strategy',
               default='private_key',
               help='Authentication driver for the API: private_key or keystone'),
    cfg.StrOpt('private_key',
               secret=True,
               default='secret',
               help='Private key for private key authentication'),
]

entity_opts = [
    cfg.IntOpt('instance_existence_threshold',
               default=900,
               help='Instance existence threshold'),
    cfg.IntOpt('volume_existence_threshold',
               default=60,
               help='Volume existence threshold'),
    cfg.ListOpt('instance_metadata',
                default=[],
                help='List of instance metadata to include from notifications'),
    cfg.ListOpt('instance_image_meta',
                default=[],
                help='List of instance image metadata to include from notifications'),
]

CONF.register_opts(database_opts, group='database')
CONF.register_opts(api_opts, group='api')
CONF.register_opts(collector_opts, group='collector')
CONF.register_opts(auth_opts, group='auth')
CONF.register_opts(keystone_opts, group='keystone_authtoken')
CONF.register_opts(entity_opts, group='entities')

logging.register_options(CONF)


def list_opts():
    return [
        ('database', database_opts),
        ('api', api_opts),
        ('collector', collector_opts),
        ('auth', auth_opts),
        ('keystone_authtoken', keystone_opts),
        ('entities', entity_opts),
    ]



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pkg_resources import get_distribution

from almanach.core.controllers import base_controller


class ApplicationController(base_controller.BaseController):

    def __init__(self, database_adapter):
        self.database_adapter = database_adapter

    def get_application_info(self):
        return {
            "info": {"version": get_distribution("almanach").version},
            "database": {"all_entities": self.database_adapter.count_entities(),
                         "active_entities": self.database_adapter.count_active_entities()}
        }



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from dateutil.parser import parse
import mock
import pytz

from almanach.core.controllers import instance_controller
from almanach.core import exception
from almanach.core import model
from almanach.storage.drivers import base_driver

from almanach.tests.unit import base
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import instance


class TestInstanceController(base.BaseTestCase):

    def setUp(self):
        super(TestInstanceController, self).setUp()
        self.config.entities.instance_image_meta = ['distro', 'version', 'os_type']
        self.database_adapter = mock.Mock(spec=base_driver.BaseDriver)
        self.controller = instance_controller.InstanceController(self.config, self.database_adapter)

    def test_instance_created(self):
        fake_instance = a(instance().with_all_dates_in_string())
        self.database_adapter.get_active_entity.side_effect = exception.EntityNotFoundException

        self.controller.create_instance(fake_instance.entity_id,
                                        fake_instance.project_id,
                                        fake_instance.start,
                                        fake_instance.flavor,
                                        fake_instance.name,
                                        fake_instance.image_meta,
                                        fake_instance.metadata)

        self.database_adapter.get_active_entity.assert_called_once_with(fake_instance.entity_id)
        self.database_adapter.insert_entity.assert_called_once()

    def test_resize_instance(self):
        fake_instance = a(instance())
        self.database_adapter.get_active_entity.return_value = fake_instance

        dates_str = "2015-10-21T16:25:00.000000Z"
        fake_instance.start = parse(dates_str)
        fake_instance.end = None
        fake_instance.last_event = parse(dates_str)

        self.controller.resize_instance(fake_instance.entity_id, "newly_flavor", dates_str)

        self.database_adapter.get_active_entity.assert_called_once_with(fake_instance.entity_id)
        self.database_adapter.close_active_entity.assert_called_once_with(fake_instance.entity_id, parse(dates_str))
        self.database_adapter.insert_entity.assert_called_once_with(fake_instance)

    def test_instance_created_but_its_an_old_event(self):
        fake_instance = a(instance()
                          .with_last_event(pytz.utc.localize(datetime(2015, 10, 21, 16, 29, 0))))
        self.database_adapter.get_active_entity.return_value = fake_instance

        self.controller.create_instance(fake_instance.entity_id, fake_instance.project_id,
                                        '2015-10-21T16:25:00.000000Z',
                                        fake_instance.flavor, fake_instance.image_meta, fake_instance.metadata)

        self.database_adapter.get_active_entity.assert_called_once_with(fake_instance.entity_id)

    def test_instance_created_but_find_garbage(self):
        fake_instance = a(instance().with_all_dates_in_string())
        self.database_adapter.get_active_entity.side_effect = exception.EntityTypeNotSupportedException

        self.controller.create_instance(fake_instance.entity_id, fake_instance.project_id, fake_instance.start,
                                        fake_instance.flavor, fake_instance.image_meta, fake_instance.metadata)

        self.database_adapter.get_active_entity.assert_called_once_with(fake_instance.entity_id)
        self.database_adapter.insert_entity.assert_called_once()

    def test_instance_deleted(self):
        self.database_adapter.has_active_entity.return_value = True

        self.controller.delete_instance("id1", "2015-10-21T16:25:00.000000Z")

        self.database_adapter.has_active_entity.assert_called_once_with("id1")
        self.database_adapter.close_active_entity.assert_called_once_with("id1", parse("2015-10-21T16:25:00.000000Z"))

    def test_instance_deleted_when_entity_not_found(self):
        self.database_adapter.has_active_entity.return_value = False

        self.assertRaises(exception.EntityNotFoundException,
                          self.controller.delete_instance,
                          "id1", "2015-10-21T16:25:00.000000Z")

        self.database_adapter.has_active_entity.assert_called_once_with("id1")

    def test_list_instances(self):
        self.database_adapter.get_all_entities_by_project.return_value = ["instance1", "instance2"]

        self.assertEqual(self.controller.list_instances("project_id", "start", "end"), ["instance1", "instance2"])

        self.database_adapter.get_all_entities_by_project.assert_called_once_with(
            "project_id", "start", "end", model.Instance.TYPE
        )

    def test_instance_rebuilded(self):
        i = a(instance())
        self.database_adapter.get_active_entity.side_effect = [i, i]
        calls = [mock.call("an_instance_id"), mock.call("an_instance_id")]

        self.controller.rebuild_instance(
            "an_instance_id",
            "2015-10-21T16:25:00.000000Z",
            dict(distro="some_distro", version="some_version", os_type="some_type")
        )
        self.controller.rebuild_instance(
            "an_instance_id",
            "2015-10-21T16:25:00.000000Z",
            dict(distro=i.image_meta['distro'], version=i.image_meta['version'], os_type=i.image_meta['os_type'])
        )

        self.database_adapter.get_active_entity.assert_has_calls(calls)
        self.database_adapter.close_active_entity.assert_called_once()
        self.database_adapter.insert_entity.assert_called_once()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils

from almanach.tests.tempest.tests.api import base


class TestServerUpdate(base.BaseAlmanachTest):

    def setUp(self):
        super(base.BaseAlmanachTest, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(TestServerUpdate, cls).resource_setup()

    def test_instance_update(self):
        server = self.get_server_creation_payload()
        self.create_server_through_api(uuidutils.generate_uuid(), server)

        update_field = json.dumps({'start_date': '2016-04-14T18:30:00.00Z',
                                   'flavor': 'NewFlavor'})
        resp, response_body = self.almanach_client.update_server(server['id'], update_field)

        updated_server = json.loads(response_body)

        self.assertEqual(server['id'], updated_server['entity_id'])
        self.assertEqual('2016-04-14 18:30:00+00:00', updated_server['start'])
        self.assertEqual('NewFlavor', updated_server['flavor'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.common import waiters
from tempest.lib.common.utils import test_utils

from almanach.tests.tempest.tests.scenario import base


class TestServerResizeScenario(base.BaseAlmanachScenarioTest):

    def test_resize_server(self):
        server, initial_flavor, resized_flavor = self._resize_server()
        self.wait_for_notification(self._check_that_a_new_entity_is_created,
                                   server)

        entities = self.get_tenant_entities(server['tenant_id'])
        self.assertEqual(2, len(entities))

        self.assertEqual(server['id'], entities[0]['entity_id'])
        self.assertEqual('instance', entities[0]['entity_type'])
        self.assertEqual(server['name'], entities[0]['name'])
        self.assertEqual(initial_flavor['name'], entities[0]['flavor'])
        self.assertIsNotNone(entities[0]['start'])
        self.assertIsNotNone(entities[0]['end'])
        self.assertEqual(dict(), entities[0]['os'])
        self.assertEqual(dict(), entities[0]['image_meta'])

        self.assertEqual(server['id'], entities[1]['entity_id'])
        self.assertEqual('instance', entities[1]['entity_type'])
        self.assertEqual(server['name'], entities[1]['name'])
        self.assertEqual(resized_flavor['name'], entities[1]['flavor'])
        self.assertIsNotNone(entities[1]['start'])
        self.assertIsNone(entities[1]['end'])
        self.assertEqual(dict(), entities[0]['os'])
        self.assertEqual(dict(), entities[0]['image_meta'])

    def _check_that_a_new_entity_is_created(self, server):
        entities = self.get_tenant_entities(server['tenant_id'])
        return len(entities) == 2

    def _resize_server(self):
        flavors = self.flavors_client.list_flavors()['flavors']
        resized_flavor = flavors[1]
        server, initial_flavor = self.create_test_server(wait_until='ACTIVE')
        self.os.servers_client.resize_server(server['id'], resized_flavor['id'])

        waiters.wait_for_server_status(self.os.servers_client, server['id'],
                                       status='VERIFY_RESIZE')

        self.os.servers_client.confirm_resize_server(server['id'])

        waiters.wait_for_server_status(self.os.servers_client, server['id'],
                                       status='ACTIVE')

        self.addCleanup(waiters.wait_for_server_termination,
                        self.os.servers_client, server['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.os.servers_client.delete_server, server['id'])

        return server, initial_flavor, resized_flavor



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from keystoneauth1 import exceptions as keystoneauth1_exceptions

from almanach.api.auth import keystone_auth
from almanach.core import exception
from almanach.tests.unit import base


class TestKeystoneAuthentication(base.BaseTestCase):

    def setUp(self):
        super(TestKeystoneAuthentication, self).setUp()
        self.session_mock = mock.patch('keystoneauth1.session.Session').start()
        self.keystone_mock = mock.patch('keystoneclient.v3.client.Client').start()

        self.validation_mock = mock.Mock()
        self.keystone_mock.return_value.tokens.validate = self.validation_mock

        self.driver = keystone_auth.KeystoneAuthentication(self.config)

    def test_with_correct_token(self):
        token = 'some keystone token'
        self.validation_mock.return_value = True
        self.driver.validate(token)
        self.validation_mock.assert_called_once_with(token)

    def test_with_invalid_token(self):
        token = 'some keystone token'
        self.validation_mock.return_value = False
        self.assertRaises(exception.AuthenticationFailureException, self.driver.validate, token)
        self.validation_mock.assert_called_once_with(token)

    def test_with_http_error(self):
        token = 'some keystone token'
        self.validation_mock.side_effect = keystoneauth1_exceptions.HttpError(message='Some Error')
        self.assertRaises(exception.AuthenticationFailureException, self.driver.validate, token)
        self.validation_mock.assert_called_once_with(token)

    def test_with_empty_token(self):
        token = None
        self.assertRaises(exception.AuthenticationFailureException, self.driver.validate, token)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils
from tempest import config
from tempest.lib.common.utils import data_utils
import tempest.test

from almanach.tests.tempest import clients

CONF = config.CONF


class BaseAlmanachTest(tempest.test.BaseTestCase):

    @classmethod
    def skip_checks(cls):
        super(BaseAlmanachTest, cls).skip_checks()

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(BaseAlmanachTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseAlmanachTest, cls).setup_clients()
        cred_provider = cls._get_credentials_provider()
        credentials = cred_provider.get_creds_by_roles(['admin']).credentials
        cls.os = clients.Manager(credentials=credentials)
        cls.almanach_client = cls.os.almanach_client

    @classmethod
    def create_server_through_api(cls, tenant_id, server):
        resp, _ = cls.almanach_client.create_server(tenant_id, json.dumps(server))
        return resp

    @staticmethod
    def get_server_creation_payload():
        server = {'id': uuidutils.generate_uuid(),
                  'created_at': '2016-01-01T18:30:00Z',
                  'name': 'api_test_instance_create',
                  'flavor': 'api_flavor',
                  'os_type': 'linux',
                  'os_distro': 'ubuntu',
                  'os_version': '14.04'}
        return server

    @staticmethod
    def get_volume_creation_payload(volume_type_name):
        return {'volume_id': uuidutils.generate_uuid(),
                'attached_to': [],
                'volume_name': 'a-test-volume',
                'volume_type': volume_type_name,
                'start': '2016-01-01T18:00:00Z',
                'size': 100}

    def create_volume_type(self):
        volume_type = {'type_id': uuidutils.generate_uuid(),
                       'type_name': data_utils.rand_name('scenario-volume-type')}
        volume_type_body = json.dumps(volume_type)
        return volume_type, self.almanach_client.create_volume_type(volume_type_body)

    def create_volume(self):
        volume_type, _ = self.create_volume_type()
        tenant_id = uuidutils.generate_uuid()
        volume = self.get_volume_creation_payload(volume_type['type_id'])
        resp, response_body = self.almanach_client.create_volume(tenant_id, json.dumps(volume))
        return resp, tenant_id, volume



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from almanach.tests.tempest.tests.scenario import base


class TestVolumeTypeScenario(base.BaseAlmanachScenarioTest):

    def test_create_volume_type(self):
        volume_type = self.create_test_volume_type()

        resp, response_body = self.almanach_client.get_volume_type(volume_type['id'])
        self.assertEqual(resp.status, 200)

        response_body = json.loads(response_body)
        self.assertIsInstance(response_body, dict)
        self.assertEqual(volume_type['id'], response_body['volume_type_id'])
        self.assertEqual(volume_type['name'], response_body['volume_type_name'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.core.controllers import application_controller
from almanach.core.controllers import entity_controller
from almanach.core.controllers import instance_controller
from almanach.core.controllers import volume_controller
from almanach.core.controllers import volume_type_controller
from almanach.storage import storage_driver


class Factory(object):

    def __init__(self, config, storage=None):
        self.config = config
        self.database_driver = None
        self.storage_driver = storage or storage_driver.StorageDriver(self.config).get_database_driver()

    def get_instance_controller(self):
        return instance_controller.InstanceController(self.config, self._get_database_driver())

    def get_volume_controller(self):
        return volume_controller.VolumeController(self.config, self._get_database_driver())

    def get_volume_type_controller(self):
        return volume_type_controller.VolumeTypeController(self._get_database_driver())

    def get_entity_controller(self):
        return entity_controller.EntityController(self._get_database_driver())

    def get_application_controller(self):
        return application_controller.ApplicationController(self._get_database_driver())

    def _get_database_driver(self):
        if not self.database_driver:
            self.database_driver = self.storage_driver
            self.database_driver.connect()
        return self.database_driver



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.core.controllers import application_controller
from almanach.core.controllers import entity_controller
from almanach.core.controllers import instance_controller
from almanach.core.controllers import volume_controller
from almanach.core.controllers import volume_type_controller
from almanach.core import factory

from almanach.tests.unit import base


class TestFactory(base.BaseTestCase):

    def setUp(self):
        super(TestFactory, self).setUp()
        self.storage = mock.Mock()
        self.factory = factory.Factory(self.config, self.storage)

    def test_get_instance_controller(self):
        self.assertIsInstance(self.factory.get_instance_controller(),
                              instance_controller.InstanceController)

    def test_get_volume_controller(self):
        self.assertIsInstance(self.factory.get_volume_controller(),
                              volume_controller.VolumeController)

    def test_get_volume_type_controller(self):
        self.assertIsInstance(self.factory.get_volume_type_controller(),
                              volume_type_controller.VolumeTypeController)

    def test_get_entity_controller(self):
        self.assertIsInstance(self.factory.get_entity_controller(),
                              entity_controller.EntityController)

    def test_get_application_controller(self):
        self.assertIsInstance(self.factory.get_application_controller(),
                              application_controller.ApplicationController)

    def test_bootstrap_storage(self):
        self.assertIsInstance(self.factory.get_instance_controller(),
                              instance_controller.InstanceController)

        self.assertIsInstance(self.factory.get_volume_controller(),
                              volume_controller.VolumeController)

        self.storage.connect.assert_called_once()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import six

from almanach.core import exception


@six.add_metaclass(abc.ABCMeta)
class Entity(object):

    def __init__(self, entity_id, project_id, start, end, last_event, name, entity_type):
        self.entity_id = entity_id
        self.project_id = project_id
        self.start = start
        self.end = end
        self.last_event = last_event
        self.name = name
        self.entity_type = entity_type

    def as_dict(self):
        return dict(
            entity_id=self.entity_id,
            project_id=self.project_id,
            start=self.start,
            end=self.end,
            last_event=self.last_event,
            name=self.name,
            entity_type=self.entity_type,
        )

    @staticmethod
    def from_dict(d):
        raise NotImplementedError

    def __eq__(self, other):
        return (other.entity_id == self.entity_id and
                other.project_id == self.project_id and
                other.start == self.start and
                other.end == self.end and
                other.last_event == self.last_event and
                other.name == self.name and
                other.entity_type == self.entity_type)

    def __ne__(self, other):
        return not self.__eq__(other)


class Instance(Entity):
    TYPE = 'instance'

    def __init__(self, entity_id, project_id, start, end, flavor, last_event, name, image_meta=None, metadata=None):
        super(Instance, self).__init__(entity_id, project_id, start, end, last_event, name, self.TYPE)
        self.flavor = flavor
        self.metadata = metadata or dict()
        self.image_meta = image_meta or dict()

        # TODO(fguillot): This attribute still used by the legacy API,
        # that should be removed when the new API v2 will be implemented
        self.os = self.image_meta

    def __eq__(self, other):
        return (super(Instance, self).__eq__(other) and
                other.flavor == self.flavor and
                other.image_meta == self.image_meta and
                other.metadata == self.metadata)

    def as_dict(self):
        d = super(Instance, self).as_dict()
        d['flavor'] = self.flavor
        d['metadata'] = self.metadata
        d['image_meta'] = self.image_meta

        # NOTE(fguillot): we keep this key for backward compatibility
        d['os'] = self.image_meta
        return d

    @staticmethod
    def from_dict(d):
        return Instance(
            entity_id=d.get('entity_id'),
            project_id=d.get('project_id'),
            start=d.get('start'),
            end=d.get('end'),
            last_event=d.get('last_event'),
            name=d.get('name'),
            flavor=d.get('flavor'),
            image_meta=d.get('os') or d.get('image_meta'),
            metadata=Instance._unserialize_metadata(d),
        )

    @staticmethod
    def _unserialize_metadata(d):
        metadata = d.get('metadata')
        if metadata:
            tmp = dict()
            for key, value in metadata.items():
                if '^' in key:
                    key = key.replace('^', '.')
                tmp[key] = value
            metadata = tmp
        return metadata

    def _serialize_metadata(self):
        tmp = dict()
        for key, value in self.metadata.items():
            if '.' in key:
                key = key.replace('.', '^')
            tmp[key] = value
        return tmp


class Volume(Entity):
    TYPE = 'volume'

    def __init__(self, entity_id, project_id, start, end, volume_type, size, last_event, name, attached_to=None):
        super(Volume, self).__init__(entity_id, project_id, start, end, last_event, name, self.TYPE)
        self.volume_type = volume_type
        self.size = size
        self.attached_to = attached_to or []

    def __eq__(self, other):
        return (super(Volume, self).__eq__(other) and
                other.volume_type == self.volume_type and
                other.size == self.size and
                other.attached_to == self.attached_to)

    def as_dict(self):
        d = super(Volume, self).as_dict()
        d['volume_type'] = self.volume_type
        d['size'] = self.size
        d['attached_to'] = self.attached_to
        return d

    @staticmethod
    def from_dict(d):
        return Volume(
            entity_id=d.get('entity_id'),
            project_id=d.get('project_id'),
            start=d.get('start'),
            end=d.get('end'),
            last_event=d.get('last_event'),
            name=d.get('name'),
            volume_type=d.get('volume_type'),
            size=d.get('size'),
            attached_to=d.get('attached_to'),
        )


class VolumeType(object):

    def __init__(self, volume_type_id, volume_type_name):
        self.volume_type_id = volume_type_id
        self.volume_type_name = volume_type_name

    def __eq__(self, other):
        return other.__dict__ == self.__dict__

    def as_dict(self):
        return dict(
            volume_type_id=self.volume_type_id,
            volume_type_name=self.volume_type_name,
        )

    @staticmethod
    def from_dict(d):
        return VolumeType(volume_type_id=d['volume_type_id'],
                          volume_type_name=d['volume_type_name'])


def get_entity_from_dict(d):
    entity_type = d.get('entity_type')
    if entity_type == Instance.TYPE:
        return Instance.from_dict(d)
    elif entity_type == Volume.TYPE:
        return Volume.from_dict(d)
    raise exception.EntityTypeNotSupportedException(
            'Unsupported entity type: "{}"'.format(entity_type))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from copy import copy
from datetime import datetime
from dateutil.parser import parse
import mock
import pytz

from almanach.core.controllers import entity_controller
from almanach.core import exception
from almanach.storage.drivers import base_driver

from almanach.tests.unit import base
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import instance


class TestEntityController(base.BaseTestCase):

    def setUp(self):
        super(TestEntityController, self).setUp()
        self.database_adapter = mock.Mock(spec=base_driver.BaseDriver)
        self.controller = entity_controller.EntityController(self.database_adapter)

    def test_update_active_instance_entity_with_a_new_flavor(self):
        flavor = u"my flavor name"
        fake_instance1 = a(instance())
        fake_instance2 = copy(fake_instance1)

        self.controller.update_active_instance_entity(
            instance_id=fake_instance1.entity_id,
            flavor=flavor,
        )

        self._expect_get_active_entity_and_update(fake_instance1, fake_instance2, flavor=flavor)

    def test_update_active_instance_entity_with_a_new_name(self):
        name = u"my instance name"
        fake_instance1 = a(instance())
        fake_instance2 = copy(fake_instance1)

        self.controller.update_active_instance_entity(
            instance_id=fake_instance1.entity_id,
            name=name,
        )

        self._expect_get_active_entity_and_update(fake_instance1, fake_instance2, name=name)

    def test_update_active_instance_entity_with_a_new_os(self):
        os = {
            "os_type": u"linux",
            "version": u"7",
            "distro": u"centos"
        }
        fake_instance1 = a(instance())
        fake_instance2 = copy(fake_instance1)

        self.controller.update_active_instance_entity(
            instance_id=fake_instance1.entity_id,
            os=os,
        )

        self._expect_get_active_entity_and_update(fake_instance1, fake_instance2, os=os)

    def test_update_active_instance_entity_with_a_new_metadata(self):
        metadata = {
            "key": "value"
        }
        fake_instance1 = a(instance())
        fake_instance2 = copy(fake_instance1)

        self.controller.update_active_instance_entity(
            instance_id=fake_instance1.entity_id,
            metadata=metadata,
        )

        self._expect_get_active_entity_and_update(fake_instance1, fake_instance2, metadata=metadata)

    def test_update_active_instance_entity_with_a_new_start_date(self):
        fake_instance1 = a(instance())
        fake_instance2 = copy(fake_instance1)

        self.controller.update_active_instance_entity(
            instance_id=fake_instance1.entity_id,
            start_date="2015-10-21T16:25:00.000000Z",
        )

        self._expect_get_active_entity_and_update(fake_instance1, fake_instance2, start="2015-10-21T16:25:00.000000Z")

    def test_update_active_instance_entity_with_a_new_end_date(self):
        fake_instance1 = a(instance())
        fake_instance2 = copy(fake_instance1)

        self.controller.update_active_instance_entity(
            instance_id=fake_instance1.entity_id,
            end_date="2015-10-21T16:25:00.000000Z",
        )

        self._expect_get_active_entity_and_update(fake_instance1, fake_instance2, end="2015-10-21T16:25:00.000000Z")

    def test_update_entity_closed_entity_flavor(self):
        start = datetime(2016, 3, 1, 0, 0, 0, 0, pytz.utc)
        end = datetime(2016, 3, 3, 0, 0, 0, 0, pytz.utc)
        flavor = 'a_new_flavor'
        fake_instance1 = a(instance().with_start(2016, 3, 1, 0, 0, 0).with_end(2016, 3, 2, 0, 0, 0))
        self.database_adapter.get_all_entities_by_id_and_date.side_effect = [[fake_instance1], [fake_instance1]]
        calls = [
            mock.call(fake_instance1.entity_id, start, end),
            mock.call(fake_instance1.entity_id, start, end)
        ]

        self.controller.update_inactive_entity(
            instance_id=fake_instance1.entity_id,
            start=start,
            end=end,
            flavor=flavor,
        )

        self.database_adapter.get_all_entities_by_id_and_date.assert_has_calls(calls)
        self.database_adapter.update_closed_entity.expect_called_once_with(
            entity=fake_instance1, data={"flavor": flavor}
        )

    def test_update_one_closed_entity_return_multiple_entities(self):
        fake_instances = [a(instance()), a(instance())]
        self.database_adapter.get_all_entities_by_id_and_date.return_value = fake_instances

        self.assertRaises(exception.MultipleEntitiesMatchingQueryException,
                          self.controller.update_inactive_entity,
                          instance_id=fake_instances[0].entity_id,
                          start=fake_instances[0].start,
                          end=fake_instances[0].end,
                          flavor=fake_instances[0].flavor)

        self.database_adapter.get_all_entities_by_id_and_date.expect_called_once_with(
            fake_instances[0].entity_id, fake_instances[0].start, fake_instances[0].end
        )

    def test_update_one_closed_entity_return_no_entity(self):
        fake_instance = a(instance())
        self.database_adapter.get_all_entities_by_id_and_date.return_value = []

        self.assertRaises(exception.EntityNotFoundException,
                          self.controller.update_inactive_entity,
                          instance_id=fake_instance.entity_id,
                          start=fake_instance.start,
                          end=fake_instance.end,
                          flavor=fake_instance.flavor)

        self.database_adapter.get_all_entities_by_id_and_date.expect_called_once_with(
            fake_instance.entity_id, fake_instance.start, fake_instance.end
        )

    def test_list_entities(self):
        self.database_adapter.get_all_entities_by_project.return_value = ["volume2", "volume3", "instance1"]

        self.assertEqual(
            self.controller.list_entities("project_id", "start", "end"),
            ["volume2", "volume3", "instance1"]
        )

        self.database_adapter.get_all_entities_by_project.expect_called_once_with(
            "project_id", "start", "end"
        )

    def test_entity_exists(self):
        entity_id = "some_entity_id"
        self.database_adapter.count_entity_entries.return_value = 1

        self.assertEqual(True, self.controller.entity_exists(entity_id))

        self.database_adapter.count_entity_entries.expect_called_once_with(entity_id=entity_id)

    def test_entity_exists_not(self):
        entity_id = "some_entity_id"
        self.database_adapter.count_entity_entries.return_value = 0

        self.assertEqual(False, self.controller.entity_exists(entity_id))

        self.database_adapter.count_entity_entries.expect_called_once_with(entity_id=entity_id)

    def test_get_all_entities_by_id(self):
        entity_id = "some_entity_id"
        fake_entity = a(instance().with_id(entity_id))
        result = [fake_entity]
        self.database_adapter.count_entity_entries.return_value = 1
        self.database_adapter.get_all_entities_by_id.return_value = result

        self.assertEqual(result, self.controller.get_all_entities_by_id(entity_id))

        self.database_adapter.count_entity_entries.expect_called_once_with(entity_id=entity_id)
        self.database_adapter.get_all_entities_by_id.expect_called_once_with(entity_id=entity_id)

    def test_get_all_entities_by_id_when_entity_not_found(self):
        entity_id = "some_entity_id"
        self.database_adapter.count_entity_entries.return_value = 0

        self.assertRaises(exception.EntityNotFoundException,
                          self.controller.get_all_entities_by_id,
                          entity_id)

        self.database_adapter.count_entity_entries.expect_called_once_with(entity_id=entity_id)

    def test_instance_updated_wrong_attributes_raises_exception(self):
        fake_instance1 = a(instance())
        self.database_adapter.get_active_entity.return_value = fake_instance1

        self.assertRaises(exception.InvalidAttributeException,
                          self.controller.update_active_instance_entity,
                          instance_id=fake_instance1.entity_id,
                          wrong_attribute="this is wrong")

        self.database_adapter.get_active_entity.expect_called_once_with(fake_instance1.entity_id)

    def _expect_get_active_entity_and_update(self, fake_instance1, fake_instance2, **kwargs):
        self.database_adapter.get_active_entity.return_value = fake_instance1
        for key, value in kwargs.items():
            if key in ['start', 'end']:
                value = parse(value)

            setattr(fake_instance2, key, value)

        self.database_adapter.get_active_entity.expect_called_once_with(fake_instance1.entity_id)
        self.database_adapter.update_active_entity.expect_called_once_with(fake_instance2)



# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.collector import notification

from almanach.tests.unit import base


class TestNotificationFilter(base.BaseTestCase):

    def setUp(self):
        super(TestNotificationFilter, self).setUp()
        self.filter = notification.NotificationFilter()

    def test_when_one_filter_returns_true(self):
        notification = mock.Mock()
        filter1 = mock.Mock()
        filter2 = mock.Mock()

        self.filter.add(filter1)
        self.filter.add(filter2)

        filter1.ignore_notification.return_value = False
        filter2.ignore_notification.return_value = True

        self.assertTrue(self.filter.ignore_notification(notification))

    def test_when_all_filters_returns_false(self):
        notification = mock.Mock()
        filter1 = mock.Mock()
        filter2 = mock.Mock()

        self.filter.add(filter1)
        self.filter.add(filter2)

        filter1.ignore_notification.return_value = False
        filter2.ignore_notification.return_value = False

        self.assertFalse(self.filter.ignore_notification(notification))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.common import waiters
from tempest.lib.common.utils import test_utils

from almanach.tests.tempest.tests.scenario import base


class TestServerCreationScenario(base.BaseAlmanachScenarioTest):

    def test_create_server(self):
        server, flavor = self.create_test_server(wait_until='ACTIVE')

        self.addCleanup(waiters.wait_for_server_termination,
                        self.os.servers_client, server['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.os.servers_client.delete_server, server['id'])

        entities = self.get_tenant_entities(server['tenant_id'])

        self.assertEqual(1, len(entities))
        self.assertEqual(server['id'], entities[0]['entity_id'])
        self.assertEqual('instance', entities[0]['entity_type'])
        self.assertEqual(server['name'], entities[0]['name'])
        self.assertEqual(flavor['name'], entities[0]['flavor'])
        self.assertIsNotNone(entities[0]['start'])
        self.assertIsNone(entities[0]['end'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from voluptuous import Invalid

from almanach.core import exception
from almanach.tests.unit.api.v1 import base_api
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import instance


class TestApiEntity(base_api.BaseApi):

    def test_update_instance_flavor_for_terminated_instance(self):
        some_new_flavor = 'some_new_flavor'
        an_instance = a(instance().
                        with_id('INSTANCE_ID').
                        with_start(2016, 3, 1, 0, 0, 0).
                        with_end(2016, 3, 3, 0, 0, 0).
                        with_flavor(some_new_flavor))
        self.entity_ctl.update_inactive_entity.return_value = an_instance
        data = dict(flavor=some_new_flavor)
        start = '2016-03-01 00:00:00.000000'
        end = '2016-03-03 00:00:00.000000'

        code, result = self.api_put(
            '/entity/instance/INSTANCE_ID',
            headers={'X-Auth-Token': 'some token value'},
            query_string={
                'start': start,
                'end': end,
            },
            data=data,
        )

        self.entity_ctl.update_inactive_entity.assert_called_once_with(
            instance_id="INSTANCE_ID",
            start=base_api.a_date_matching(start),
            end=base_api.a_date_matching(end),
            flavor=some_new_flavor
        )
        self.assertEqual(code, 200)
        self.assertIn('entity_id', result)
        self.assertIn('flavor', result)
        self.assertEqual(result['flavor'], some_new_flavor)

    def test_update_instance_entity_with_a_new_start_date(self):
        data = {
            "start_date": "2014-01-01 00:00:00.0000",
        }
        an_instance = a(instance().with_id('INSTANCE_ID').with_start(2014, 1, 1, 0, 0, 0))
        self.entity_ctl.update_active_instance_entity.return_value = an_instance

        code, result = self.api_put(
            '/entity/instance/INSTANCE_ID',
            headers={'X-Auth-Token': 'some token value'},
            data=data,
        )

        self.entity_ctl.update_active_instance_entity.assert_called_once_with(
            instance_id="INSTANCE_ID",
            start_date=data["start_date"]
        )
        self.assertEqual(code, 200)
        self.assertIn('entity_id', result)
        self.assertIn('start', result)
        self.assertIn('end', result)
        self.assertEqual(result['start'], "2014-01-01 00:00:00+00:00")

    def test_update_active_instance_entity_with_bad_payload(self):
        self.entity_ctl.update_active_instance_entity.side_effect = ValueError(
            'Expecting object: line 1 column 15 (char 14)'
        )
        instance_id = 'INSTANCE_ID'
        data = {
            'flavor': 'A_FLAVOR',
        }

        code, result = self.api_put(
                '/entity/instance/INSTANCE_ID',
                data=data,
                headers={'X-Auth-Token': 'some token value'}
        )

        self.entity_ctl.update_active_instance_entity.assert_called_once_with(instance_id=instance_id, **data)
        self.assertIn("error", result)
        self.assertEqual(result["error"], 'Invalid parameter or payload')
        self.assertEqual(code, 400)

    def test_update_active_instance_entity_with_wrong_attribute_raise_exception(self):
        errors = [
            Invalid(message="error message1", path=["my_attribute1"]),
            Invalid(message="error message2", path=["my_attribute2"]),
        ]
        self.entity_ctl.update_active_instance_entity.side_effect = exception.InvalidAttributeException(errors)

        formatted_errors = {
            "my_attribute1": "error message1",
            "my_attribute2": "error message2",
        }

        instance_id = 'INSTANCE_ID'
        data = {
            'flavor': 'A_FLAVOR',
        }

        code, result = self.api_put(
            '/entity/instance/INSTANCE_ID',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.entity_ctl.update_active_instance_entity.assert_called_once_with(instance_id=instance_id, **data)
        self.assertIn("error", result)
        self.assertEqual(result['error'], formatted_errors)
        self.assertEqual(code, 400)

    def test_entity_head_with_existing_entity(self):
        self.entity_ctl.entity_exists.return_value = True
        entity_id = "entity_id"

        code, result = self.api_head('/entity/{id}'.format(id=entity_id), headers={'X-Auth-Token': 'some token value'})

        self.entity_ctl.entity_exists.assert_called_once_with(entity_id=entity_id)
        self.assertEqual(code, 200)

    def test_entity_head_with_nonexistent_entity(self):
        self.entity_ctl.entity_exists.return_value = False
        entity_id = "entity_id"

        code, result = self.api_head('/entity/{id}'.format(id=entity_id), headers={'X-Auth-Token': 'some token value'})

        self.entity_ctl.entity_exists.assert_called_once_with(entity_id=entity_id)
        self.assertEqual(code, 404)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils

from almanach.tests.tempest.tests.api import base


class TestVolumeDetach(base.BaseAlmanachTest):

    @classmethod
    def resource_setup(cls):
        super(TestVolumeDetach, cls).resource_setup()

    def test_detach(self):
        resp, tenant_id, volume = self.create_volume()
        self.assertEqual(resp.status, 201)

        attached_to_id = uuidutils.generate_uuid()
        date = '2016-01-01 18:30:00+00:00'
        attach_data = {'date': '2016-01-01T18:30:00Z', 'attachments': [attached_to_id]}
        resp, _ = self.almanach_client.attach_volume(volume['volume_id'], json.dumps(attach_data))
        self.assertEqual(resp.status, 200)
        detach_data = {'date': '2016-01-01T18:40:00Z',
                       'attachments': []}

        resp, _ = self.almanach_client.detach_volume(volume['volume_id'], json.dumps(detach_data))
        self.assertEqual(resp.status, 200)

        resp, response_body = self.almanach_client.get_tenant_entities(tenant_id)
        response_body = json.loads(response_body)

        self.assertEqual(resp.status, 200)
        self.assertIsInstance(response_body, list)
        self.assertEqual(3, len(response_body))

        un_attached_volumes = [v for v in response_body if v['attached_to'] == []]
        attached_volume = [v for v in response_body if v['attached_to'] != []][0]

        self.assertEqual(volume['volume_id'], attached_volume['entity_id'])
        self.assertEqual(volume['volume_id'], un_attached_volumes[0]['entity_id'])
        self.assertEqual(volume['volume_id'], un_attached_volumes[1]['entity_id'])
        self.assertEqual('volume', attached_volume['entity_type'])
        self.assertEqual('volume', un_attached_volumes[0]['entity_type'])
        self.assertEqual('volume', un_attached_volumes[1]['entity_type'])
        self.assertIsNotNone(attached_volume['end'])
        self.assertIn(None, [un_attached_volumes[0]['end'], un_attached_volumes[1]['end']])
        self.assertIn(date, [un_attached_volumes[0]['end'], un_attached_volumes[1]['end']])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.collector.handlers import instance_handler
from almanach.core import exception
from almanach.tests.unit import base
from almanach.tests.unit.builders import notification as builder


class TestInstanceHandler(base.BaseTestCase):

    def setUp(self):
        super(TestInstanceHandler, self).setUp()
        self.controller = mock.Mock()
        self.on_delete_filter = mock.Mock()
        self.on_delete_filter.ignore_notification.return_value = False
        self.instance_handler = instance_handler.InstanceHandler(self.controller, self.on_delete_filter)

    def test_instance_created(self):
        notification = builder.InstanceNotificationBuilder()\
            .with_event_type('compute.instance.create.end')\
            .with_image_meta('os_type', 'linux')\
            .with_image_meta('distro', 'ubuntu')\
            .with_image_meta('version', '16.04')\
            .build()

        self.instance_handler.handle_events(notification)

        self.controller.create_instance.assert_called_once_with(
            instance_id=notification.payload['instance_id'],
            tenant_id=notification.payload['tenant_id'],
            create_date=notification.payload['created_at'],
            name=notification.payload['hostname'],
            flavor=notification.payload['instance_type'],
            image_meta=notification.payload['image_meta'],
            metadata=notification.payload['metadata'],
        )

    def test_instance_deleted(self):
        notification = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.delete.end') \
            .with_payload_value('terminated_at', 'a_date') \
            .build()

        self.instance_handler.handle_events(notification)

        self.controller.delete_instance.assert_called_once_with(
            notification.payload['instance_id'],
            notification.payload['terminated_at']
        )

    def test_instance_deleted_but_never_created(self):
        notification = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.delete.end') \
            .with_payload_value('terminated_at', 'a_date') \
            .build()

        self.controller.delete_instance.side_effect = exception.EntityNotFoundException('Instance not found')

        self.assertRaises(exception.EntityNotFoundException,
                          self.instance_handler.handle_events, notification)

        self.controller.delete_instance.assert_called_once_with(
            notification.payload['instance_id'],
            notification.payload['terminated_at']
        )

    def test_instance_in_error_deleted(self):
        self.on_delete_filter.ignore_notification.return_value = True

        notification = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.delete.end') \
            .with_payload_value('terminated_at', 'a_date') \
            .with_payload_value('state', 'error') \
            .build()

        self.controller.delete_instance.side_effect = exception.EntityNotFoundException('Instance not found')
        self.instance_handler.handle_events(notification)

        self.controller.delete_instance.assert_called_once_with(
            notification.payload['instance_id'],
            notification.payload['terminated_at']
        )

    def test_instance_resized(self):
        notification = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.resize.confirm.end') \
            .with_context_value('timestamp', 'a_date') \
            .build()

        self.instance_handler.handle_events(notification)

        self.controller.resize_instance.assert_called_once_with(
            notification.payload['instance_id'],
            notification.payload['instance_type'],
            notification.context.get("timestamp")
        )

    def test_instance_rebuild(self):
        notification = builder.InstanceNotificationBuilder() \
            .with_event_type('compute.instance.rebuild.end') \
            .with_context_value('timestamp', 'a_date') \
            .with_image_meta('os_type', 'linux') \
            .with_image_meta('distro', 'ubuntu') \
            .with_image_meta('version', '16.04') \
            .build()

        self.instance_handler.handle_events(notification)

        self.controller.rebuild_instance.assert_called_once_with(
            instance_id=notification.payload['instance_id'],
            rebuild_date=notification.context.get("timestamp"),
            image_meta=notification.payload['image_meta'],
        )



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.core.controllers import volume_type_controller

from almanach.tests.unit import base
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import volume_type


class TestVolumeTypeController(base.BaseTestCase):

    def setUp(self):
        super(TestVolumeTypeController, self).setUp()
        self.database_adapter = mock.Mock()
        self.controller = volume_type_controller.VolumeTypeController(self.database_adapter)

    def test_create_volume_type(self):
        fake_volume_type = a(volume_type())
        self.controller.create_volume_type(fake_volume_type.volume_type_id, fake_volume_type.volume_type_name)
        self.database_adapter.insert_volume_type.assert_called_once_with(fake_volume_type)

    def test_get_volume_type(self):
        some_volume = a(volume_type())
        self.database_adapter.get_volume_type.return_value = some_volume

        a_volume_type = self.controller.get_volume_type(some_volume.volume_type_id)

        self.assertEqual(some_volume, a_volume_type)
        self.database_adapter.get_volume_type.assert_called_once_with(some_volume.volume_type_id)

    def test_delete_volume_type(self):
        some_volume = a(volume_type())
        self.controller.delete_volume_type(some_volume.volume_type_id)
        self.database_adapter.delete_volume_type.assert_called_once_with(some_volume.volume_type_id)

    def test_list_volume_types(self):
        some_volumes = [a(volume_type()), a(volume_type())]
        self.database_adapter.list_volume_types.return_value = some_volumes
        self.assertEqual(len(self.controller.list_volume_types()), 2)
        self.database_adapter.list_volume_types.assert_called_once()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

ServiceAvailableGroup = [
    cfg.BoolOpt("almanach",
                default=True,
                help="Whether or not Almanach is expected to be available"),
]

usage_group = cfg.OptGroup(name="almanach",
                           title="Usage Service Options")

UsageGroup = [
    cfg.StrOpt("region",
               default="",
               help="The usage region name to use. If empty, the value "
                    "of identity.region is used instead. If no such region "
                    "is found in the service catalog, the first found one is "
                    "used."),
    cfg.StrOpt("catalog_type",
               default="usage",
               help="Catalog type for Almanach."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the usage service.")
]



# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from dateutil import parser
import pytz

from almanach.core import exception


class DateHelper(object):

    @staticmethod
    def parse(value):
        try:
            dt = value if isinstance(value, datetime) else parser.parse(value)
            return DateHelper._normalize_timezone(dt)
        except (TypeError, ValueError):
            raise exception.DateFormatException()

    @staticmethod
    def is_within_range(d1, d2, delta):
        diff = (d2 - d1).total_seconds()
        return abs(diff) < delta

    @staticmethod
    def _normalize_timezone(dt):
        try:
            return dt.astimezone(pytz.utc)
        except ValueError:
            return dt.replace(tzinfo=pytz.utc)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.common import waiters
from tempest.lib.common.utils import test_utils

from almanach.tests.tempest.tests.scenario import base


class TestVolumeResizeScenario(base.BaseAlmanachScenarioTest):

    def test_resize_volume(self):
        volume = self._resize_volume()

        entities = self.get_tenant_entities(volume['os-vol-tenant-attr:tenant_id'])
        self.assertEqual(2, len(entities))

        self.assertEqual(volume['id'], entities[0]['entity_id'])
        self.assertEqual(volume['volume_type'], entities[0]['volume_type'])
        self.assertEqual(1, entities[0]['size'])
        self.assertIsNotNone(entities[0]['start'])
        self.assertIsNotNone(entities[0]['end'])

        self.assertEqual(volume['id'], entities[1]['entity_id'])
        self.assertEqual(volume['volume_type'], entities[1]['volume_type'])
        self.assertEqual(2, entities[1]['size'])
        self.assertIsNotNone(entities[1]['start'])
        self.assertIsNone(entities[1]['end'])

    def _resize_volume(self):
        volume = self.create_test_volume(size=1)
        self.volumes_client.extend_volume(volume['id'], new_size=2)

        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        self.addCleanup(self.volumes_client.wait_for_resource_deletion,
                        volume['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.volumes_client.delete_volume, volume['id'])
        return volume



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
import oslo_messaging

LOG = logging.getLogger(__name__)


class MessagingFactory(object):

    def __init__(self, config):
        self.config = config

    def get_listeners(self, handler):
        listeners = []
        targets = [
            oslo_messaging.Target(topic=self.config.collector.topic),
        ]

        for url in self.config.collector.transport_url:
            LOG.debug('Creating listener for %s', url)
            transport = self._get_transport(url)
            listeners.append(oslo_messaging.get_notification_listener(transport=transport,
                                                                      targets=targets,
                                                                      endpoints=[handler],
                                                                      executor='threading'))
        return listeners

    def get_notifier(self):
        transport = self._get_transport(self.config.collector.transport_url[0])
        return oslo_messaging.Notifier(transport, publisher_id='almanach.collector',
                                       topics=[self.config.collector.topic], driver='messagingv2')

    def _get_transport(self, url):
        return oslo_messaging.get_notification_transport(self.config, url=url)



# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from almanach.collector.filters import base_filter

LOG = logging.getLogger(__name__)


class ErroredInstanceFilter(base_filter.BaseFilter):

    def ignore_notification(self, notification):
        if notification.payload.get('state') == 'error':
            LOG.info('Instance deletion event ignored because instance %s was badly created',
                     notification.payload.get('instance_id'))
            return True
        return False



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from dateutil import parser as date_parser
import six

import pytz

from almanach.core import exception


@six.add_metaclass(abc.ABCMeta)
class BaseController(object):

    def _validate_and_parse_date(self, date):
        try:
            date = date_parser.parse(date)
            return self._localize_date(date)
        except (TypeError, ValueError):
            raise exception.DateFormatException()

    @staticmethod
    def _localize_date(date):
        try:
            return pytz.utc.localize(date)
        except ValueError:
            return date

    def _fresher_entity_exists(self, entity_id, date):
        try:
            entity = self.database_adapter.get_active_entity(entity_id)
            if entity and entity.last_event > date:
                return True
        except exception.EntityNotFoundException:
            pass
        except exception.EntityTypeNotSupportedException:
            pass
        return False

    def _transform_attribute_to_match_entity_attribute(self, **kwargs):
        entity = {}
        for attribute, key in dict(start="start_date", end="end_date").items():
            if kwargs.get(key):
                entity[attribute] = self._validate_and_parse_date(kwargs.get(key))

        for attribute in ["name", "flavor", "os", "metadata"]:
            if kwargs.get(attribute):
                entity[attribute] = kwargs.get(attribute)
        return entity



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime

import pytz

from almanach.core import exception
from almanach.core import model

from almanach.tests.unit import base


class TestModel(base.BaseTestCase):

    def test_instance_serialize(self):
        instance = model.Instance(
            entity_id='instance_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            flavor='flavor_id',
            image_meta=dict(os_type='linux', distro='Ubuntu', version='16.04'),
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='hostname',
            metadata={'some_key': 'some.value', 'another^key': 'another.value'},
        )

        entry = instance.as_dict()
        self.assertEqual('instance_id', entry['entity_id'])
        self.assertEqual('project_id', entry['project_id'])
        self.assertEqual('instance', entry['entity_type'])
        self.assertEqual('hostname', entry['name'])
        self.assertEqual('flavor_id', entry['flavor'])
        self.assertEqual('linux', entry['os']['os_type'])
        self.assertEqual('Ubuntu', entry['os']['distro'])
        self.assertEqual('16.04', entry['os']['version'])
        self.assertEqual('linux', entry['image_meta']['os_type'])
        self.assertEqual('Ubuntu', entry['image_meta']['distro'])
        self.assertEqual('16.04', entry['image_meta']['version'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), entry['last_event'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), entry['start'])
        self.assertIsNone(entry['end'])

    def test_instance_unserialize(self):
        entry = {
            'entity_id': 'instance_id',
            'entity_type': 'instance',
            'project_id': 'project_id',
            'start': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'end': None,
            'last_event': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'flavor': 'flavor_id',
            'image_meta': {
                'os_type': 'linux',
                'distro': 'Ubuntu',
                'version': '16.04',
            },
            'name': 'hostname'
        }

        instance = model.get_entity_from_dict(entry)
        self.assertEqual('instance_id', instance.entity_id)
        self.assertEqual('project_id', instance.project_id)
        self.assertEqual('instance', instance.entity_type)
        self.assertEqual('hostname', instance.name)
        self.assertEqual('flavor_id', instance.flavor)
        self.assertEqual('linux', instance.image_meta['os_type'])
        self.assertEqual('Ubuntu', instance.image_meta['distro'])
        self.assertEqual('16.04', instance.image_meta['version'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), instance.last_event)
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), instance.start)
        self.assertIsNone(instance.end)

    def test_instance_unserialize_with_legacy_os(self):
        entry = {
            'entity_id': 'instance_id',
            'entity_type': 'instance',
            'project_id': 'project_id',
            'start': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'end': None,
            'last_event': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'flavor': 'flavor_id',
            'os': {
                'os_type': 'linux',
                'distro': 'Ubuntu',
                'version': '16.04',
            },
            'name': 'hostname'
        }

        instance = model.get_entity_from_dict(entry)
        self.assertEqual('instance_id', instance.entity_id)
        self.assertEqual('project_id', instance.project_id)
        self.assertEqual('instance', instance.entity_type)
        self.assertEqual('hostname', instance.name)
        self.assertEqual('flavor_id', instance.flavor)
        self.assertEqual('linux', instance.image_meta['os_type'])
        self.assertEqual('Ubuntu', instance.image_meta['distro'])
        self.assertEqual('16.04', instance.image_meta['version'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), instance.last_event)
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), instance.start)
        self.assertIsNone(instance.end)

    def test_instance_unserialize_with_both_keys(self):
        entry = {
            'entity_id': 'instance_id',
            'entity_type': 'instance',
            'project_id': 'project_id',
            'start': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'end': None,
            'last_event': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'flavor': 'flavor_id',
            'os': {
                'os_type': 'linux',
                'distro': 'Ubuntu',
                'version': '16.04',
            },
            'image_meta': {
                'os_type': 'linux',
                'distro': 'Ubuntu',
                'version': '16.04',
            },
            'name': 'hostname'
        }

        instance = model.get_entity_from_dict(entry)
        self.assertEqual('instance_id', instance.entity_id)
        self.assertEqual('project_id', instance.project_id)
        self.assertEqual('instance', instance.entity_type)
        self.assertEqual('hostname', instance.name)
        self.assertEqual('flavor_id', instance.flavor)
        self.assertEqual('linux', instance.image_meta['os_type'])
        self.assertEqual('Ubuntu', instance.image_meta['distro'])
        self.assertEqual('16.04', instance.image_meta['version'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), instance.last_event)
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), instance.start)
        self.assertIsNone(instance.end)

    def test_instance_comparison(self):
        instance1 = model.Instance(
            entity_id='instance_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            flavor='flavor_id',
            image_meta=dict(os_type='linux', distro='Ubuntu', version='16.04'),
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='hostname',
            metadata={'some_key': 'some.value', 'another^key': 'another.value'},
        )

        instance2 = model.Instance(
            entity_id='instance_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            flavor='flavor_id',
            image_meta=dict(os_type='linux', distro='Ubuntu', version='16.04'),
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='hostname',
            metadata={'some_key': 'some.value', 'another^key': 'another.value'},
        )

        # different image properties
        instance3 = model.Instance(
            entity_id='instance_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            flavor='flavor_id',
            image_meta=dict(os_type='linux', distro='Centos', version='7'),
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='hostname',
            metadata={'some_key': 'some.value', 'another^key': 'another.value'},
        )

        # different flavor
        instance4 = model.Instance(
            entity_id='instance_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            flavor='another_flavor',
            image_meta=dict(os_type='linux', distro='Ubuntu', version='16.04'),
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='hostname',
            metadata={'some_key': 'some.value', 'another^key': 'another.value'},
        )

        # different metadata
        instance5 = model.Instance(
            entity_id='instance_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            flavor='flavor_id',
            image_meta=dict(os_type='linux', distro='Ubuntu', version='16.04'),
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='hostname',
            metadata=dict(),
        )

        self.assertTrue(instance1 == instance2)
        self.assertTrue(instance1 != instance3)
        self.assertTrue(instance1 != instance4)
        self.assertTrue(instance1 != instance5)

    def test_volume_serialize(self):
        volume = model.Volume(
            entity_id='volume_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='volume_name',
            volume_type='volume_type_id',
            size=2,
            attached_to=['instance_id1', 'instance_id2'],
        )

        entry = volume.as_dict()
        self.assertEqual('volume_id', entry['entity_id'])
        self.assertEqual('project_id', entry['project_id'])
        self.assertEqual('volume', entry['entity_type'])
        self.assertEqual('volume_name', entry['name'])
        self.assertEqual('volume_type_id', entry['volume_type'])
        self.assertEqual(2, entry['size'])
        self.assertEqual(['instance_id1', 'instance_id2'], entry['attached_to'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), entry['last_event'])
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), entry['start'])
        self.assertIsNone(entry['end'])

    def test_volume_unserialize(self):
        entry = {
            'entity_id': 'volume_id',
            'entity_type': 'volume',
            'project_id': 'project_id',
            'start': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'end': None,
            'last_event': datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            'volume_type': 'volume_type_id',
            'name': 'volume_name',
            'size': 2,
        }

        volume = model.get_entity_from_dict(entry)
        self.assertEqual('volume_id', volume.entity_id)
        self.assertEqual('project_id', volume.project_id)
        self.assertEqual('volume', volume.entity_type)
        self.assertEqual('volume_name', volume.name)
        self.assertEqual(2, volume.size)
        self.assertEqual([], volume.attached_to)
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), volume.last_event)
        self.assertEqual(datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc), volume.start)
        self.assertIsNone(volume.end)

    def test_volume_comparison(self):
        volume1 = model.Volume(
            entity_id='volume_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='volume_name',
            volume_type='volume_type_id',
            size=2,
            attached_to=['instance_id1', 'instance_id2'],
        )

        volume2 = model.Volume(
            entity_id='volume_id',
            project_id='project_id',
            start=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            end=None,
            last_event=datetime(2014, 1, 1, 0, 0, 0, 0, pytz.utc),
            name='volume_name',
            volume_type='volume_type_id',
            size=2,
            attached_to=['instance_id1', 'instance_id2'],
        )

        self.assertTrue(volume1 == volume2)

        volume2.volume_type = 'another_volume_type'
        self.assertFalse(volume1 == volume2)

        volume2.volume_type = 'volume_type_id'
        volume2.size = 3
        self.assertFalse(volume1 == volume2)

        volume2.volume_type = 'volume_type_id'
        volume2.size = 2
        volume2.attached_to = []
        self.assertFalse(volume1 == volume2)

    def test_volume_type_serialize(self):
        volume_type = model.VolumeType(
            volume_type_id='id',
            volume_type_name='name',
        )

        entry = volume_type.as_dict()
        self.assertEqual('id', entry['volume_type_id'])
        self.assertEqual('name', entry['volume_type_name'])

    def test_volume_type_unserialize(self):
        entry = dict(volume_type_id='id', volume_type_name='name')

        volume_type = model.VolumeType.from_dict(entry)
        self.assertEqual('id', volume_type.volume_type_id)
        self.assertEqual('name', volume_type.volume_type_name)

    def test_volume_type_comparison(self):
        volume_type1 = model.VolumeType(volume_type_id='id', volume_type_name='name')
        volume_type2 = model.VolumeType(volume_type_id='id2', volume_type_name='name2')
        volume_type3 = model.VolumeType(volume_type_id='id', volume_type_name='name')

        self.assertTrue(volume_type1 != volume_type2)
        self.assertTrue(volume_type1 == volume_type3)

    def test_unserialize_unknown_entity(self):
        self.assertRaises(exception.EntityTypeNotSupportedException,
                          model.get_entity_from_dict,
                          dict(entity_type='unknown'))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest import config
from tempest.lib.common import rest_client

CONF = config.CONF


class AlmanachClient(rest_client.RestClient):

    def __init__(self, auth_provider):
        super(AlmanachClient, self).__init__(
                auth_provider,
                CONF.almanach.catalog_type,
                CONF.almanach.region or CONF.identity.region,
                endpoint_type=CONF.almanach.endpoint_type)

    def get_version(self):
        resp, response_body = self.get('info')
        return resp, response_body

    def create_server(self, tenant_id, body):
        resp, response_body = self.post('/project/{}/instance'.format(tenant_id), body=body)
        return resp, response_body

    def delete_server(self, instance_id, body):
        resp, response_body = self.delete('/instance/{}'.format(instance_id), body=body)
        return resp, response_body

    def get_volume_type(self, volume_type_id):
        resp, response_body = self.get('volume_type/{}'.format(volume_type_id))
        return resp, response_body

    def create_volume_type(self, body):
        resp, response_body = self.post('/volume_type', body)
        return resp, response_body

    def create_volume(self, tenant_id, body):
        url = '/project/{}/volume'.format(tenant_id)
        resp, response_body = self.post(url, body)
        return resp, response_body

    def delete_volume(self, volume_id, body):
        resp, response_body = self.delete('/volume/{}'.format(volume_id), body=body)
        return resp, response_body

    def resize_volume(self, volume_id, body):
        resp, response_body = self.put('/volume/{}/resize'.format(volume_id), body)
        return resp, response_body

    def attach_volume(self, volume_id, body):
        resp, response_body = self.put('/volume/{}/attach'.format(volume_id), body)
        return resp, response_body

    def detach_volume(self, volume_id, body):
        resp, response_body = self.put('/volume/{}/detach'.format(volume_id), body)
        return resp, response_body

    def get_tenant_entities(self, tenant_id):
        url = 'project/{}/entities?start=2016-01-01%2000:00:00.000'.format(tenant_id)
        resp, response_body = self.get(url)
        return resp, response_body

    def update_server(self, instance_id, body):
        url = '/entity/instance/{}'.format(instance_id)
        resp, response_body = self.put(url, body)
        return resp, response_body

    def rebuild(self, instance_id, body):
        update_instance_rebuild_query = "/instance/{}/rebuild".format(instance_id)
        resp, response_body = self.put(update_instance_rebuild_query, body)
        return resp, response_body

    def resize(self, instance_id, body):
        url = "/instance/{}/resize".format(instance_id)
        resp, response_body = self.put(url, body)
        return resp, response_body



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

import oslo_messaging

from almanach.collector import messaging
from almanach.tests.unit import base


class TestMessagingFactory(base.BaseTestCase):

    def setUp(self):
        super(TestMessagingFactory, self).setUp()
        self.factory = messaging.MessagingFactory(self.config)

    def test_get_listeners_with_one_listener(self):
        self.config.collector.transport_url = ['rabbit://guest:guest@localhost:5672']
        listeners = self.factory.get_listeners(mock.Mock())
        self.assertEqual(1, len(listeners))

    def test_get_listeners_with_multiple_listener(self):
        self.config.collector.transport_url = ['rabbit://guest:guest@localhost:5672',
                                               'rabbit://guest:guest@localhost:5673']
        listeners = self.factory.get_listeners(mock.Mock())
        self.assertEqual(2, len(listeners))

    def test_get_notifier(self):
        self.assertIsInstance(self.factory.get_notifier(), oslo_messaging.Notifier)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.api.auth import private_key_auth
from almanach.core import exception
from almanach.tests.unit import base


class TestPrivateKeyAuthentication(base.BaseTestCase):

    def setUp(self):
        super(TestPrivateKeyAuthentication, self).setUp()
        self.auth_backend = private_key_auth.PrivateKeyAuthentication("my token")

    def test_with_correct_token(self):
        self.assertEqual(self.auth_backend.validate("my token"), True)

    def test_with_invalid_token(self):
        self.assertRaises(exception.AuthenticationFailureException, self.auth_backend.validate, "bad token")

    def test_with_empty_token(self):
        self.assertRaises(exception.AuthenticationFailureException, self.auth_backend.validate, None)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from almanach.collector.handlers import base_handler
from almanach.core import exception

LOG = logging.getLogger(__name__)


class InstanceHandler(base_handler.BaseHandler):

    def __init__(self, controller, on_delete_filter):
        super(InstanceHandler, self).__init__()
        self.controller = controller
        self.on_delete_filter = on_delete_filter

    def handle_events(self, notification):
        if notification.event_type == "compute.instance.create.end":
            self._on_instance_created(notification)
        elif notification.event_type == "compute.instance.delete.end":
            self._on_instance_deleted(notification)
        elif notification.event_type == "compute.instance.resize.confirm.end":
            self._on_instance_resized(notification)
        elif notification.event_type == "compute.instance.rebuild.end":
            self._on_instance_rebuilt(notification)

    def _on_instance_created(self, notification):
        self.controller.create_instance(
            instance_id=notification.payload.get("instance_id"),
            tenant_id=notification.payload.get("tenant_id"),
            create_date=notification.payload.get("created_at"),
            name=notification.payload.get("hostname"),
            flavor=notification.payload.get("instance_type"),
            image_meta=notification.payload.get("image_meta"),
            metadata=notification.payload.get("metadata"),
        )

    def _on_instance_deleted(self, notification):
        date = notification.payload.get("terminated_at")
        instance_id = notification.payload.get("instance_id")

        try:
            self.controller.delete_instance(instance_id, date)
        except exception.EntityNotFoundException as e:
            if not self.on_delete_filter.ignore_notification(notification):
                raise e

    def _on_instance_resized(self, notification):
        date = notification.context.get("timestamp")
        flavor = notification.payload.get("instance_type")
        instance_id = notification.payload.get("instance_id")
        self.controller.resize_instance(instance_id, flavor, date)

    def _on_instance_rebuilt(self, notification):
        date = notification.context.get("timestamp")
        instance_id = notification.payload.get("instance_id")
        self.controller.rebuild_instance(
            instance_id=instance_id,
            rebuild_date=date,
            image_meta=notification.payload.get("image_meta")
        )



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.core import exception
from almanach.tests.unit.api.v1 import base_api
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import volume_type


class TestApiVolumeType(base_api.BaseApi):

    def test_get_volume_types(self):
        self.volume_type_ctl.list_volume_types.return_value = [
            a(volume_type().with_volume_type_name('some_volume_type_name'))
        ]

        code, result = self.api_get('/volume_types', headers={'X-Auth-Token': 'some token value'})

        self.volume_type_ctl.list_volume_types.assert_called_once()
        self.assertEqual(code, 200)
        self.assertEqual(len(result), 1)
        self.assertIn('volume_type_name', result[0])
        self.assertEqual(result[0]['volume_type_name'], 'some_volume_type_name')

    def test_successful_volume_type_create(self):
        data = dict(
            type_id='A_VOLUME_TYPE_ID',
            type_name="A_VOLUME_TYPE_NAME"
        )

        code, result = self.api_post('/volume_type', data=data, headers={'X-Auth-Token': 'some token value'})

        self.volume_type_ctl.create_volume_type.assert_called_once_with(
            volume_type_id=data['type_id'],
            volume_type_name=data['type_name']
        )
        self.assertEqual(code, 201)

    def test_volume_type_create_missing_a_param_returns_bad_request_code(self):
        data = dict(type_name="A_VOLUME_TYPE_NAME")

        code, result = self.api_post('/volume_type', data=data, headers={'X-Auth-Token': 'some token value'})

        self.volume_type_ctl.create_volume_type.assert_not_called()
        self.assertEqual(result["error"], "The 'type_id' param is mandatory for the request you have made.")
        self.assertEqual(code, 400)

    def test_volume_type_delete_with_authentication(self):
        code, result = self.api_delete('/volume_type/A_VOLUME_TYPE_ID', headers={'X-Auth-Token': 'some token value'})

        self.volume_type_ctl.delete_volume_type.assert_called_once_with('A_VOLUME_TYPE_ID')
        self.assertEqual(code, 202)

    def test_volume_type_delete_not_in_database(self):
        self.volume_type_ctl.delete_volume_type.side_effect = exception.AlmanachException("An exception occurred")

        code, result = self.api_delete('/volume_type/A_VOLUME_TYPE_ID', headers={'X-Auth-Token': 'some token value'})

        self.volume_type_ctl.delete_volume_type.assert_called_once_with('A_VOLUME_TYPE_ID')
        self.assertIn("An exception occurred", result["error"])
        self.assertEqual(code, 500)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils

from almanach.tests.tempest.tests.api import base


class TestServerRebuild(base.BaseAlmanachTest):

    def setUp(self):
        super(base.BaseAlmanachTest, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(TestServerRebuild, cls).resource_setup()

    def test_server_rebuild(self):
        tenant_id = uuidutils.generate_uuid()
        server = self.get_server_creation_payload()
        self.create_server_through_api(tenant_id, server)

        data = {
            'distro': 'debian',
            'version': '8.0',
            'os_type': 'linux',
            'rebuild_date': '2016-01-01T18:50:00Z'
        }

        self.almanach_client.rebuild(server['id'],
                                     json.dumps(data))

        resp, response_body = self.almanach_client.get_tenant_entities(tenant_id)

        entities = json.loads(response_body)
        self.assertIsInstance(entities, list)
        self.assertEqual(2, len(entities))

        rebuilt_server, initial_server = sorted(entities, key=lambda k: k['end'] if k['end'] is not None else '')

        self.assertEqual(server['id'], initial_server['entity_id'])
        self.assertIsNotNone(initial_server['end'])

        self.assertEqual(server['os_distro'], initial_server['os']['distro'])
        self.assertEqual(server['os_version'], initial_server['os']['version'])
        self.assertEqual(server['os_type'], initial_server['os']['os_type'])

        self.assertEqual(server['os_distro'], initial_server['image_meta']['distro'])
        self.assertEqual(server['os_version'], initial_server['image_meta']['version'])
        self.assertEqual(server['os_type'], initial_server['image_meta']['os_type'])

        self.assertEqual(server['id'], rebuilt_server['entity_id'])
        self.assertIsNone(rebuilt_server['end'])

        self.assertEqual(data['distro'], rebuilt_server['os']['distro'])
        self.assertEqual(data['version'], rebuilt_server['os']['version'])
        self.assertEqual(data['os_type'], rebuilt_server['os']['os_type'])

        self.assertEqual(data['distro'], rebuilt_server['image_meta']['distro'])
        self.assertEqual(data['version'], rebuilt_server['image_meta']['version'])
        self.assertEqual(data['os_type'], rebuilt_server['image_meta']['os_type'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.core.controllers import base_controller
from almanach.core import exception
from almanach.core import model

LOG = log.getLogger(__name__)


class InstanceController(base_controller.BaseController):

    def __init__(self, config, database_adapter):
        self.config = config
        self.database_adapter = database_adapter

    def create_instance(self, instance_id, tenant_id, create_date, name, flavor, image_meta=None, metadata=None):
        create_date = self._validate_and_parse_date(create_date)
        image_meta = self._filter_image_meta(image_meta)
        LOG.info("Instance %s created (tenant %s; flavor %s; image_meta %s) on %s",
                 instance_id, tenant_id, flavor, image_meta, create_date)

        if self._fresher_entity_exists(instance_id, create_date):
            LOG.warning("instance %s already exists with a more recent entry", instance_id)
            return

        entity = model.Instance(
            entity_id=instance_id,
            project_id=tenant_id,
            last_event=create_date,
            start=create_date,
            end=None,
            name=name,
            flavor=flavor,
            image_meta=image_meta,
            metadata=self._filter_metadata(metadata))

        self.database_adapter.insert_entity(entity)

    def delete_instance(self, instance_id, delete_date):
        if not self.database_adapter.has_active_entity(instance_id):
            raise exception.EntityNotFoundException(
                    "InstanceId: {0} Not Found".format(instance_id))

        delete_date = self._validate_and_parse_date(delete_date)
        LOG.info("Instance %s deleted on %s", instance_id, delete_date)
        self.database_adapter.close_active_entity(instance_id, delete_date)

    def resize_instance(self, instance_id, flavor, resize_date):
        resize_date = self._validate_and_parse_date(resize_date)
        LOG.info("Instance %s resized to flavor %s on %s", instance_id, flavor, resize_date)
        try:
            instance = self.database_adapter.get_active_entity(instance_id)
            if flavor != instance.flavor:
                self.database_adapter.close_active_entity(instance_id, resize_date)
                instance.flavor = flavor
                instance.start = resize_date
                instance.end = None
                instance.last_event = resize_date
                self.database_adapter.insert_entity(instance)
        except exception.EntityNotFoundException as e:
            LOG.error("Trying to resize an instance with id '%s' not in the database yet.", instance_id)
            raise e

    def rebuild_instance(self, instance_id, rebuild_date, image_meta):
        rebuild_date = self._validate_and_parse_date(rebuild_date)
        instance = self.database_adapter.get_active_entity(instance_id)
        image_meta = self._filter_image_meta(image_meta)
        LOG.info("Instance %s rebuilt for tenant %s with %s on %s",
                 instance_id, instance.project_id, image_meta, rebuild_date)

        if instance.image_meta != image_meta:
            self.database_adapter.close_active_entity(instance_id, rebuild_date)
            instance.image_meta = image_meta
            instance.start = rebuild_date
            instance.end = None
            instance.last_event = rebuild_date
            self.database_adapter.insert_entity(instance)

    def list_instances(self, project_id, start, end):
        return self.database_adapter.get_all_entities_by_project(project_id, start, end, model.Instance.TYPE)

    def _filter_metadata(self, metadata):
        return self._filter(metadata, self.config.entities.instance_metadata)

    def _filter_image_meta(self, image_meta):
        return self._filter(image_meta, self.config.entities.instance_image_meta)

    @staticmethod
    def _filter(d, whitelist):
        if d:
            return {key: value for key, value in d.items() if key in whitelist}
        return {}
